local ADDON_NAME = ...

-----------------------------
-- Frame

local StacksFrame = CreateFrame("Frame", nil, UIParent)

local StacksFrame_OnUpdate
do
	local debuffName = GetSpellInfo(62489) -- "Blue Pyrite"

	--returns a color in a gradiant from green (1) to red (0)
	local function grad(ratio)
		ratio = math.min(1, ratio)
		--(1 + ((0) - 1) * ratio) * 255, (0 + ((1) - 0) * ratio) * 255), 0
		return string.format("%02x%02x00", (1 - ratio) * 255, ratio * 255)
	end

	function StacksFrame_OnUpdate()
		local power = UnitPower("vehicle")
		if power >= 50 then
			StacksFrame.background:SetVertexColor(1, 1, 1, 1)
		else
			--ramp up alpha wrt energy from 30 to 50, providing a visual jump (.68 -> 1) at 50
			StacksFrame.background:SetVertexColor(1, 1, 1, math.max(0.3, (power - 15) * 2 / 100))
		end

		local _, _, _, count, _, _, expires = UnitDebuff("target", debuffName, nil, "PLAYER") --UnitDebuff("target", 1, "PLAYER") --could break if you debuff maly as you fall?
		if expires then
			--color the text based on the remaining time
			local remaining = expires - GetTime()
			StacksFrame.time:SetFormattedText("|cff%s%0.1f s|r", grad(remaining / 15), remaining)
			StacksFrame.time:Show()
		else
			StacksFrame.time:Hide()
		end

		StacksFrame.count:SetFormattedText("|cffffffff%d|r", count or 0)
	end
end

local function SetupFrame()
	local frame = StacksFrame
	frame:SetWidth(64)
	frame:SetHeight(64)
	frame:ClearAllPoints()
	frame:SetPoint(PyriteStacksDB.point or "CENTER", PyriteStacksDB.x or 0, PyriteStacksDB.y or 0)

	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", frame.StartMoving)
	frame:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		local point, _, _, x, y = self:GetPoint(1)
		PyriteStacksDB.point = point
		PyriteStacksDB.x = x
		PyriteStacksDB.y = y
	end)

	local texture = frame:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(frame)
	texture:SetTexture("Interface\\Icons\\Ability_Vehicle_LiquidPyrite _blue")
	frame.background = texture

	local text1 = frame:CreateFontString(nil, "OVERLAY")
	text1:ClearAllPoints()
	text1:SetPoint("TOP", frame, "BOTTOM", 0, 0)
	text1:SetJustifyH("CENTER")
	text1:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	frame.time = text1

	local text2 = frame:CreateFontString(nil, "OVERLAY")
	text2:SetWidth(64)
	text2:SetHeight(64)
	text2:ClearAllPoints()
	text2:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
	text2:SetJustifyH("CENTER")
	text2:SetJustifyV("MIDDLE")
	text2:SetFont("Fonts\\FRIZQT__.TTF", 36, "OUTLINE")
	text2:SetText("|cFFFFFFFF0|r")
	frame.count = text2

	--ripped from PitBull_ComboPoints by ckknight
	local comboPoints = CreateFrame("Frame", nil, frame)
	comboPoints:SetWidth(60)
	comboPoints:SetHeight(12)
	comboPoints:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -2, 0)
	for i = 1, 5 do
		local combo = comboPoints:CreateTexture(nil, "OVERLAY")
		combo:SetTexture("Interface\\AddOns\\PyriteStacks\\combo")
		combo:SetWidth(12)
		combo:SetHeight(12)
		combo:ClearAllPoints()
		if i == 1 then
			combo:SetPoint("RIGHT", comboPoints, "RIGHT")
		else
			combo:SetPoint("RIGHT", comboPoints["combo"..(i-1)], "LEFT")
		end
		combo:Hide()
		comboPoints["combo"..i] = combo
	end
	frame.comboPoints = comboPoints

	frame:SetScript("OnUpdate", StacksFrame_OnUpdate)
end


-----------------------------
-- Events

StacksFrame:SetScript("OnEvent", function(self, event, ...)
	if self[event] then self[event](self, ...) end
end)

function StacksFrame:UNIT_ENTERED_VEHICLE(unit)
	if unit == "player" then
		local guid = tonumber(string.sub(UnitGUID("vehicle") or "", 7, 10), 16)
		if guid == 33109 then
			if not StacksFrame.background then
				SetupFrame()
			end
			self:Show()
			self:RegisterEvent("PLAYER_TARGET_CHANGED")
			self:RegisterEvent("UNIT_COMBO_POINTS")
			self:RegisterEvent("UNIT_EXITED_VEHICLE")
		end
	end
end

function StacksFrame:UNIT_EXITED_VEHICLE(unit)
	if unit == "player" and self:IsShown() then
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:UnregisterEvent("UNIT_COMBO_POINTS")
		self:UnregisterEvent("UNIT_EXITED_VEHICLE")
		self:Hide()
	end
end

function StacksFrame:UNIT_COMBO_POINTS()
	if not UnitInVehicle("player") then
		self:UNIT_EXITED_VEHICLE("player")
		return
	end

	local comboPoints = GetComboPoints("vehicle", "target")
	for i = 1, 5 do
		if comboPoints >= i then
			StacksFrame.comboPoints["combo"..i]:Show()
		else
			StacksFrame.comboPoints["combo"..i]:Hide()
		end
	end
	StacksFrame.comboPoints:Show()
end
StacksFrame.PLAYER_TARGET_CHANGED = StacksFrame.UNIT_COMBO_POINTS


function StacksFrame:ADDON_LOADED(addon)
	if addon ~= ADDON_NAME then return end

	PyriteStacksDB = PyriteStacksDB or {}
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")

	if IsLoggedIn() then
		self:UNIT_ENTERED_VEHICLE("player")
	end

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
end
StacksFrame:RegisterEvent("ADDON_LOADED")


SLASH_PyriteStacks1 = "/PyriteStacks"
SlashCmdList["PyriteStacks"] = function()
	StacksFrame:UNIT_ENTERED_VEHICLE("player")
end
